@@ -22,10 +22,10 @@
  */
 
 #include <gtk/gtkstock.h>
+#include <gtk/gtkwindow.h>
 #include <libgnomedb/gnome-db-error-dialog.h>
 #include <libgnomedb/gnome-db-util.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <bonobo/bonobo-i18n.h>
+#include "gnome-db-i18n.h"
 
 struct _GnomeDbErrorDialogPrivate {
 	GnomeDbError *error_widget;
@@ -194,8 +194,8 @@ gnome_db_error_dialog_new (gchar *title)
 	dialog = g_object_new (GNOME_DB_TYPE_ERROR_DIALOG, NULL);
 	if (title)
 		gnome_db_error_dialog_set_title (dialog, title);
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (dialog),
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 	if (title)
 		gnome_db_error_dialog_set_title (dialog, title);
 
