@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include <glib/gfileutils.h>
 #include <libgda/gda-util.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkradiobutton.h>
@@ -28,6 +28,7 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeview.h>
+#include <gtk/gtkwindow.h>
 #include <gobject/gmarshal.h>
 #include <libgnomedb/gnome-db-form.h>
 #include <libgnomedb/gnome-db-gray-bar.h>
@@ -36,9 +37,10 @@
 #include <libgnomedb/gnome-db-model.h>
 #include <libgnomedb/gnome-db-stock.h>
 #include <libgnomedb/gnome-db-table-editor.h>
+#ifdef BUILD_WITH_GNOME
 #include <libgnomeui/gnome-file-entry.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <bonobo/bonobo-i18n.h>
+#endif
+#include "gnome-db-i18n.h"
 #include "libgnomedb-private.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
@@ -50,11 +52,12 @@ struct _GnomeDbGridPrivate {
 	GtkWidget *title;
 	GtkWidget *scroll;
 	GtkWidget *tree_view;
-	GtkWidget *popup_menu;
 
 	gchar *title_string;
 	gboolean show_title;
 	GtkSelectionMode selection_mode;
+
+	GHashTable *row_data;
 };
 
 static void gnome_db_grid_class_init   (GnomeDbGridClass *klass);
@@ -72,6 +75,8 @@ static void gnome_db_grid_finalize   (GObject *object);
 enum {
 	ROW_SELECTED,
 	SELECTION_CLEARED,
+	DOUBLE_CLICKED,
+	CREATE_POPUP_MENU,
 	LAST_SIGNAL
 };
 
@@ -104,8 +109,8 @@ menu_describe_cb (GtkWidget *widget, gpointer user_data)
 		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (grid))), 0,
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 		NULL);
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (dialog),
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 
 	description = gnome_db_table_editor_new ();
 	gtk_widget_show (description);
@@ -126,6 +131,68 @@ menu_hide_columns_cb (GtkWidget *widget, gpointer user_data)
 	gnome_db_grid_hide_column_titles (grid);
 }
 
+#ifdef BUILD_WITH_GNOME
+static void
+save_as_response_cb (GtkDialog *dialog, guint response_id, gpointer user_data)
+{
+	GtkWidget *to_tab;
+	GtkWidget *to_comma;
+	GtkWidget *to_xml;
+	GtkWidget *filename;
+	GnomeDbGrid *grid = (GnomeDbGrid *) user_data;
+
+	if (response_id == GTK_RESPONSE_OK) {
+		gchar *body;
+		gchar *path;
+
+		to_tab = g_object_get_data (G_OBJECT (dialog), "to_tab");
+		to_comma = g_object_get_data (G_OBJECT (dialog), "to_comma");
+		to_xml = g_object_get_data (G_OBJECT (dialog), "to_xml");
+		filename = g_object_get_data (G_OBJECT (dialog), "filename");
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_tab)))
+			body = gda_data_model_to_tab_separated (grid->priv->model);
+		else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_comma)))
+			body = gda_data_model_to_comma_separated (grid->priv->model);
+		else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_xml)))
+			body = gda_data_model_to_xml (grid->priv->model, TRUE);
+		else
+			body = NULL;
+
+		if (body) {
+			path = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (filename), FALSE);
+			if (path) {
+				if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+					if (!gnome_db_yes_no_dialog (GTK_WIDGET (dialog),
+								     _("File %s already exists\n"
+								       "Do you want to overwrite it?"), path)) {
+						g_free (body);
+						g_free (path);
+						return;
+					}
+					    
+				}
+
+				if (!gda_file_save (path, body, strlen (body))) {
+					gnome_db_show_error (_("Could not save file %s"), path);
+					g_free (body);
+					g_free (path);
+					return;
+				}
+				g_free (path);
+			} else {
+				gnome_db_show_error (_("You must specify a file name"));
+				g_free (body);
+				return;
+			}
+			g_free (body);
+		} else
+			gnome_db_show_error (_("Got empty file while converting the data"));
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 static void
 menu_save_as_cb (GtkWidget *widget, gpointer user_data)
 {
@@ -148,56 +215,38 @@ menu_save_as_cb (GtkWidget *widget, gpointer user_data)
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_OK, GTK_RESPONSE_OK,
 		NULL);
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (dialog),
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 	table = gnome_db_new_table_widget (6, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, 0);
 
 	label = gnome_db_new_label_widget (_("Save to"));
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
 	filename = gnome_db_new_file_entry_widget ("gnome-db-grid-save-as", _("Select File"));
+	g_object_set_data (G_OBJECT (dialog), "filename", filename);
 	gtk_table_attach (GTK_TABLE (table), filename, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
 
 	to_tab = gnome_db_new_radio_button_widget (NULL, _("Tab-delimited file"));
+	g_object_set_data (G_OBJECT (dialog), "to_tab", to_tab);
 	gtk_table_attach (GTK_TABLE (table), to_tab, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
+
 	to_comma = gnome_db_new_radio_button_widget (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (to_tab)),
 		_("Comma-delimited file"));
+	g_object_set_data (G_OBJECT (dialog), "to_comma", to_comma);
 	gtk_table_attach (GTK_TABLE (table), to_comma, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+
 	to_xml = gnome_db_new_radio_button_widget (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (to_tab)),
 		_("XML file"));
+	g_object_set_data (G_OBJECT (dialog), "to_xml", to_xml);
 	gtk_table_attach (GTK_TABLE (table), to_xml, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
 
 	/* run the dialog */
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
-		gchar *body;
-		gchar *path;
-
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_tab)))
-			body = gda_data_model_to_tab_separated (grid->priv->model);
-		else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_comma)))
-			body = gda_data_model_to_comma_separated (grid->priv->model);
-		else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (to_xml)))
-			body = gda_data_model_to_xml (grid->priv->model, FALSE);
-		else
-			body = NULL;
-
-		if (body) {
-			path = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (filename), FALSE);
-			if (path) {
-				if (!gda_file_save (path, body, strlen (body)))
-					gnome_db_show_error (_("Could not save file %s"), path);
-				g_free (path);
-			}
-			else
-				gnome_db_show_error (_("You must specify a file name"));
-			g_free (body);
-		}
-	}
-
-	gtk_widget_destroy (dialog);
+	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (save_as_response_cb), grid);
+	gtk_widget_show (dialog);
 }
+#endif
 
 static void
 menu_select_all_cb (GtkWidget *widget, gpointer user_data)
@@ -241,8 +290,8 @@ menu_view_detail_cb (GtkWidget *widget, gpointer user_data)
 		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (grid))), 0,
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 		NULL);
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (dialog),
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 
 	form = gnome_db_form_new ();
 	gtk_widget_show (form);
@@ -255,6 +304,75 @@ menu_view_detail_cb (GtkWidget *widget, gpointer user_data)
 	gtk_widget_destroy (dialog);
 }
 
+static void
+model_row_removed_cb (GdaDataModel *model, gint row, gpointer user_data)
+{
+	gint i, rows;
+	gpointer key, value;
+	GnomeDbGrid *grid = (GnomeDbGrid *) user_data;
+
+	g_return_if_fail (GNOME_DB_IS_GRID (grid));
+
+	rows = gda_data_model_get_n_rows (grid->priv->model);
+	for (i = row; i <= rows; i++) {
+		if (g_hash_table_lookup_extended (grid->priv->row_data, GINT_TO_POINTER (i + 1), &key, &value)) {
+			g_hash_table_remove (grid->priv->row_data, GINT_TO_POINTER (i + 1));
+			if (i > row) {
+				/* update the rows */
+				g_hash_table_insert (grid->priv->row_data, GINT_TO_POINTER (i), value);
+			}
+		}
+	}
+}
+
+static void
+add_popup_menu_item (GtkMenu *menu, const gchar *label, gboolean pixmap, GCallback cb_func, gpointer user_data)
+{
+	GtkWidget *item;
+
+	if (pixmap)
+		item = gtk_image_menu_item_new_from_stock (label, NULL);
+	else
+		item = gtk_menu_item_new_with_label (label);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (cb_func), user_data);
+	gtk_menu_append (GTK_MENU (menu), item);
+	gtk_widget_show (item);
+
+}
+
+static gint
+popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+	GtkWidget *menu;
+	GnomeDbGrid *grid = (GnomeDbGrid *) user_data;
+
+	g_return_if_fail (GNOME_DB_IS_GRID (grid));
+
+	if (event->button != 3)
+		return FALSE;
+
+	/* create the menu */
+	menu = gtk_menu_new ();
+	add_popup_menu_item (GTK_MENU (menu), _("Select All"), FALSE, menu_select_all_cb, grid);
+	add_popup_menu_item (GTK_MENU (menu), _("Clear selection"), FALSE, menu_unselect_all_cb, grid);
+	add_popup_menu_item (GTK_MENU (menu), _("Show columns"), FALSE, menu_show_columns_cb, grid);
+	add_popup_menu_item (GTK_MENU (menu), _("Hide columns"), FALSE, menu_hide_columns_cb, grid);
+#ifdef BUILD_WITH_GNOME
+	add_popup_menu_item (GTK_MENU (menu), GTK_STOCK_SAVE_AS, TRUE, menu_save_as_cb, grid);
+#endif
+#if 0
+	add_popup_menu_item (GTK_MENU (menu), _("Describe"), menu_describe_cb, grid);
+	add_popup_menu_item (GTK_MENU (menu), _("View detail..."), menu_view_detail_cb, grid);
+#endif
+
+	/* allow listeners to add their custom menu items */
+	g_signal_emit (G_OBJECT (grid), gnome_db_grid_signals[CREATE_POPUP_MENU], 0, GTK_MENU (menu));
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
+
+	return TRUE;
+}
+
 static void
 selection_foreach (GtkTreeModel *tree_model,
 		   GtkTreePath *path,
@@ -305,52 +423,45 @@ selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
 	}
 }
 
+static void
+tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path,
+			    GtkTreeViewColumn *column, gpointer user_data)
+{
+	GtkTreeIter iter;
+	DataModelInfo *info = NULL;
+	GnomeDbGrid *grid = (GnomeDbGrid *) user_data;
+
+	g_return_if_fail (GNOME_DB_IS_GRID (grid));
+
+	if (!gtk_tree_model_get_iter (gtk_tree_view_get_model (grid->priv->tree_view), &iter, path))
+		return;
+
+	gtk_tree_model_get (gtk_tree_view_get_model (grid->priv->tree_view), &iter, 0, &info, -1);
+	if (!info)
+		return;
+
+	g_signal_emit (G_OBJECT (grid), gnome_db_grid_signals[DOUBLE_CLICKED], 0, info->row);
+}
+
 /*
  * Private functions
  */
 
-static GnomeUIInfo grid_popup_menu[] = {
-	{ GNOME_APP_UI_ITEM, N_("Select All"), N_("Select all items in list"),
-	  menu_select_all_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
-	  NULL, 0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Unselect All"), N_("Unselect all selected items in list"),
-	  menu_unselect_all_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
-	  NULL, 0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Show columns"), N_("Display column headers"),
-	  menu_show_columns_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
-	  NULL, 0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("Hide columns"), N_("Hide column headers"),
-	  menu_hide_columns_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
-	  NULL, 0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("Save as..."), N_("Export data to different formats"),
-          menu_save_as_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-          GTK_STOCK_SAVE_AS, 0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
-#if 0
-	{ GNOME_APP_UI_ITEM, N_("Describe..."), N_("Describe current data set"),
-          menu_describe_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-          GNOME_DB_STOCK_TABLES, 0, 0, NULL },
-#endif
-	{ GNOME_APP_UI_ITEM, N_("View detail..."), N_("View current data in more detail"),
-          menu_view_detail_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-          GTK_STOCK_PROPERTIES, 0, 0, NULL },
-	GNOMEUIINFO_END
-};
-
 static void
 setup_grid (GnomeDbGrid *grid)
 {
 	GtkTreeSelection *selection;
 
+	g_signal_connect (G_OBJECT (grid->priv->tree_view), "row_activated",
+			  G_CALLBACK (tree_view_row_activated_cb), grid);
+	g_signal_connect (G_OBJECT (grid->priv->tree_view), "button_press_event",
+			  G_CALLBACK (popup_button_pressed_cb), grid);
+
 	/* setup selection stuff */
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->tree_view));
 	gtk_tree_selection_set_mode (selection, grid->priv->selection_mode);
 	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (selection_changed_cb), grid);
-
-	grid->priv->popup_menu = gnome_db_new_popup_menu (grid->priv->tree_view,
-							  grid_popup_menu, grid);
 }
 
 /*
@@ -394,6 +505,22 @@ gnome_db_grid_class_init (GnomeDbGridClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	gnome_db_grid_signals[DOUBLE_CLICKED] =
+		g_signal_new ("double_clicked",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GnomeDbGridClass, double_clicked),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE, 1, G_TYPE_INT);
+	gnome_db_grid_signals[CREATE_POPUP_MENU] =
+		g_signal_new ("create_popup_menu",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GnomeDbGridClass, create_popup_menu),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	/* clipboard atom */
 	if (!clipboard_atom)
@@ -407,6 +534,7 @@ gnome_db_grid_init (GnomeDbGrid *grid, GnomeDbGridClass *klass)
 	grid->priv = g_new0 (GnomeDbGridPrivate, 1);
 	grid->priv->show_title = FALSE;
 	grid->priv->title_string = g_strdup (_("Title"));
+	grid->priv->row_data = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	/* set up widgets */
 	grid->priv->title = gnome_db_gray_bar_new (grid->priv->title_string);
@@ -475,6 +603,15 @@ gnome_db_grid_finalize (GObject *object)
 		grid->priv->title_string = NULL;
 	}
 
+	if (grid->priv->model) {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (grid->priv->model), model_row_removed_cb, grid);
+		g_object_unref (G_OBJECT (grid->priv->model));
+		grid->priv->model = NULL;
+	}
+
+	g_hash_table_destroy (grid->priv->row_data);
+	grid->priv->row_data = NULL;
+
 	g_free (grid->priv);
 	grid->priv = NULL;
 
@@ -606,6 +743,11 @@ gnome_db_grid_get_model (GnomeDbGrid *grid)
 	return grid->priv->model;
 }
 
+static void
+dummy_remove_hash (gpointer key, gpointer value, gpointer user_data)
+{
+}
+
 /**
  * gnome_db_grid_set_model
  */
@@ -614,13 +756,20 @@ gnome_db_grid_set_model (GnomeDbGrid *grid, GdaDataModel *model)
 {
 	g_return_if_fail (GNOME_DB_IS_GRID (grid));
 
-	if (GDA_IS_DATA_MODEL (grid->priv->model))
+	/* clean up current data */
+	if (GDA_IS_DATA_MODEL (grid->priv->model)) {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (grid->priv->model), model_row_removed_cb, grid);
 		g_object_unref (G_OBJECT (grid->priv->model));
+	}
+
+	g_hash_table_foreach_remove (grid->priv->row_data, (GHRFunc) dummy_remove_hash, NULL);
 
 	grid->priv->model = model;
-	if (GDA_IS_DATA_MODEL (model))
+	if (GDA_IS_DATA_MODEL (model)) {
 		g_object_ref (G_OBJECT (model));
-	else {
+		g_signal_connect (G_OBJECT (grid->priv->model), "row_removed",
+				  G_CALLBACK (model_row_removed_cb), grid);
+	} else {
 		/* just empty the grid and return */
 		gtk_tree_view_set_model (GTK_TREE_VIEW (grid->priv->tree_view), NULL);
 		return;
@@ -779,3 +928,82 @@ gnome_db_grid_show_column_titles (GnomeDbGrid *grid)
 	g_return_if_fail (GNOME_DB_IS_GRID (grid));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (grid->priv->tree_view), TRUE);
 }
+
+/**
+ * gnome_db_grid_get_row_data
+ * @grid: a #GnomeDbGrid widget.
+ * @row: the row to retrieve data for.
+ *
+ * Get the data associated (see #gnome_db_grid_set_row_data) with the given row.
+ *
+ * Returns: the data associated with the given row.
+ */
+gpointer
+gnome_db_grid_get_row_data (GnomeDbGrid *grid, gint row)
+{
+	g_return_val_if_fail (GNOME_DB_IS_GRID (grid), NULL);
+	return g_hash_table_lookup (grid->priv->row_data, GINT_TO_POINTER (row + 1));
+}
+
+/**
+ * gnome_db_grid_set_row_data
+ * @grid: a #GnomeDbGrid widget.
+ * @row: the row to set the data for.
+ * @data: the data.
+ *
+ * Associate a given data pointer with the given row in the grid widget. This
+ * allows you to store context-specific data for each row on the grid.
+ */
+void
+gnome_db_grid_set_row_data (GnomeDbGrid *grid, gint row, gpointer data)
+{
+	g_return_if_fail (GNOME_DB_IS_GRID (grid));
+	g_return_if_fail (row >= 0);
+	g_return_if_fail (grid->priv->model != NULL);
+	g_return_if_fail (row < gda_data_model_get_n_rows (grid->priv->model));
+
+	if (g_hash_table_lookup (grid->priv->row_data, GINT_TO_POINTER (row + 1)))
+		g_hash_table_remove (grid->priv->row_data, GINT_TO_POINTER (row + 1));
+
+	if (data)
+		g_hash_table_insert (grid->priv->row_data, GINT_TO_POINTER (row + 1), data);
+}
+
+typedef struct {
+	gint row;
+	gpointer search_data;
+} find_data_t;
+
+static void
+find_by_data_in_hash (gpointer key, gpointer value, gpointer data)
+{
+	find_data_t *fdata = data;
+
+	if (fdata->row == -1) {
+		if (fdata->search_data == value)
+			fdata->row = GPOINTER_TO_INT (key) - 1;
+	}
+}
+
+/**
+ * gnome_db_grid_find_row_from_data
+ * @grid: a #GnomeDbGrid widget.
+ * @data: data to search for.
+ *
+ * Find the row number for the row to which the given data is associated.
+ *
+ * Returns: the row number if found, or -1 if not.
+ */
+gint
+gnome_db_grid_find_row_from_data (GnomeDbGrid *grid, gpointer data)
+{
+	find_data_t fdata;
+
+	g_return_val_if_fail (GNOME_DB_IS_GRID (grid), -1);
+	g_return_val_if_fail (data != NULL, -1);
+
+	fdata.row = -1;
+	fdata.search_data = data;
+	g_hash_table_foreach (grid->priv->row_data, (GHFunc) find_by_data_in_hash, &fdata);
+	return fdata.row;
+}
