@@ -50,16 +50,21 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkvpaned.h>
 #include <gtk/gtkvseparator.h>
+#include <gtk/gtkwindow.h>
+#ifdef BUILD_WITH_GNOME
 #include <libgnomedb/gnome-db-browser.h>
+#endif
 #include <libgnomedb/gnome-db-grid.h>
 #include <libgnomedb/gnome-db-util.h>
+#ifdef BUILD_WITH_GNOME
 #include <libgnomeui/gnome-file-entry.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <bonobo/bonobo-i18n.h>
+#endif
+#include "gnome-db-i18n.h"
 
 /**
  * gnome_db_new_browser_widget
  */
+#ifdef BUILD_WITH_GNOME
 GtkWidget *
 gnome_db_new_browser_widget (void)
 {
@@ -70,6 +75,7 @@ gnome_db_new_browser_widget (void)
 
 	return browser;
 }
+#endif
 
 /**
  * gnome_db_new_button_widget
@@ -150,6 +156,7 @@ gnome_db_new_entry_widget (gint max_length, gboolean editable)
 	return entry;
 }
 
+#ifdef BUILD_WITH_GNOME
 /**
  * gnome_db_new_file_entry_widget
  */
@@ -163,6 +170,7 @@ gnome_db_new_file_entry_widget (const gchar *history_id, const gchar *title)
 
 	return entry;
 }
+#endif
 
 /**
  * gnome_db_new_frame_widget
@@ -298,6 +306,7 @@ gnome_db_new_option_menu_widget (void)
 	return option_menu;
 }
 
+#ifdef BUILD_WITH_GNOME
 /**
  * gnome_db_new_popup_menu
  */
@@ -314,6 +323,7 @@ gnome_db_new_popup_menu (GtkWidget *parent, GnomeUIInfo *ui_info, gpointer user_
 
         return popup_menu;
 }
+#endif
 
 /**
  * gnome_db_new_radio_button_widget
@@ -752,8 +762,8 @@ gnome_db_yes_no_dialog (GtkWidget *parent, const gchar *format, ...)
 					 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_YES_NO, sz);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Question"));
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (dialog),
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
 		retval = TRUE;
@@ -808,3 +818,40 @@ gnome_db_show_message (const gchar *format, ...)
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
+
+void
+gnome_db_push_cursor_busy (GtkWidget *window)
+{
+        int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "GnomeDb:WindowBusy"));
+                                                                                
+        busy ++;
+                                                                                
+        if (busy == 1) {
+                gtk_widget_set_sensitive (window, FALSE);
+                if (window->window != NULL) {
+                        GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
+                        gdk_window_set_cursor (window->window, cursor);
+                        gdk_cursor_unref (cursor);
+                        gdk_flush ();
+                }
+        }
+                                                                                
+        g_object_set_data (G_OBJECT (window), "GnomeDb:WindowBusy", GINT_TO_POINTER (busy));
+}
+                                                                                
+void
+gnome_db_pop_cursor_busy (GtkWidget *window)
+{
+        int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "GnomeDb:WindowBusy"));
+        busy --;
+                                                                                
+        if (busy <= 0) {
+                gtk_widget_set_sensitive (window, TRUE);
+                if (window->window != NULL)
+                        gdk_window_set_cursor (window->window, NULL);
+                g_object_set_data (G_OBJECT (window),
+                                   "Panel:WindowBusy", NULL);
+        } else {
+                g_object_set_data (G_OBJECT (window), "Panel:WindowBusy", GINT_TO_POINTER (busy));
+        }
+}
